import java.net.*;
import java.io.*;

/**
 * Implementation der Serverklasse.
 * Erzeugt einen Serversocket auf Port 6666 und lsst diesen als Thread laufen.
 * Der Server wartet fr 5ms auf eine Verbindung, 
 * sollte diese nicht auftreten schlft der Thread fr 10sek vor dem nchsten Versuch.  
 * @author      Jonas Temmen
 */
public class MyXmlServer extends Thread{
	/** Der ServerSocket */
	private ServerSocket mySocketServer;
	/** Der Socket*/
	private Socket mySocket;
	/** Instanz des SMSReaders*/
	private SMSReader mySMS;
	/** Instanz des Outputstreams*/
	private OutputStream out;
	/** Indikator ob der Server grade luft und Daten sendet*/
	private boolean isRunning=true;
	
	/**
	 * Erzeugt ein neuen Server an Port 6666
	 *
	 * @param  printMsgsTo  Die JTextarea an die die Nachrichten des Servers wieder gibt
	 * @throws IOException Falls ein Fehler beim Erzeugen des Servers auftritt bzw der Port belegt ist
	 */
	public MyXmlServer(){
		try{
			//Server instanziieren
			this.mySocketServer = new ServerSocket(6666);
			//neuen Reader erzeugen
			this.mySMS=new SMSReader();
		}
		catch(IOException ioEx){
			ioEx.printStackTrace();
			System.exit(0);
		}
	}
	
	/**
	 * Run Methode des Threads
	 *
	 * @throws SocketTimeoutException Falls sich kein Client nach 5ms verbunden hat 
	 * @throws IOException Im Falle eines Fehlers bei der Ein- oder Ausgabe
	 * @throws InterruptedException Falls der Thread unterbrochen wurde
	 */
	public void run()  {  
		//solange der Server als "luft" definiert ist
		while(this.isRunning){
			try{
				//falls es noch keinen Socket gibt
				if(this.mySocket==null){
					//Timeout fr verbindungsversuch setzen
					mySocketServer.setSoTimeout(5);
					//auf verbindung warten (Methode blockiert dabei...daher das timeout)
					this.mySocket = mySocketServer.accept();
					this.mySocket.setTcpNoDelay(true);
					//Outputstream holen
					this.out = mySocket.getOutputStream();
					
				} //fals es einen gibt
				else{
					//daten senden
					this.serverSend();
				}
				//100ms schlafen
				sleep(100);
			}
			//keine verbindung des clients erfolgt
			catch(SocketTimeoutException ste){
				try{
					sleep(10000);
				}catch(Exception ex){}
			}
			catch(IOException ioex){}
			catch(InterruptedException iEx){
				System.exit(0);
			}
		}
		
	}
	/**
	 * Sendet Daten des SMSReaders ber den outputstream.
	 *
	 * @throws SocketException Falls der Client die Verbindung unterbrochen hat
	 * @throws IOException Im Falle eines Fehlers bei der Ein- oder Ausgabe
	 * @see SMSReader
	 */
	private void serverSend(){
		try{
			//hole daten vom Reader
			String data=this.mySMS.getSMSDataAsSimpleString();
			//auf dem Stream schreiben
			this.out.write(data.getBytes(), 0, data.length());
			//0 Byte senden (quasi flush
			this.out.write(0);
		}
		catch(SocketException se){
			System.exit(0);
		}
		catch(IOException ioEx){
			System.exit(0);
		}
		
	}
}